package com.ruoyi.wx.web.service.impl;

import cn.hutool.core.date.DateUtil;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.WxUser;
import com.ruoyi.common.core.domain.model.LoginWxUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.framework.recovery.domain.WxCoupons;
import com.ruoyi.framework.recovery.domain.WxCredit;
import com.ruoyi.framework.recovery.domain.dto.UserCouponsDto;
import com.ruoyi.framework.recovery.domain.vo.CreditVo;
import com.ruoyi.framework.recovery.domain.vo.UserCouponsVo;
import com.ruoyi.framework.recovery.mapper.WxCouponsMapper;
import com.ruoyi.framework.recovery.mapper.WxCreditMapper;
import com.ruoyi.framework.recovery.mapper.WxUserMapper;
import com.ruoyi.wx.web.domain.dto.CreditDto;
import com.ruoyi.wx.web.domain.vo.WxCreditVo;
import com.ruoyi.wx.web.domain.vo.WxUserVo;
import com.ruoyi.wx.web.service.IWxPersonCenterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class WxPersonCenterServiceImpl implements IWxPersonCenterService {
    @Autowired
    private WxCreditMapper wxCreditMapper;

    @Autowired
    private WxCouponsMapper couponsMapper;

    @Autowired
    private WxUserMapper wxUserMapper;

    @Override
    public AjaxResult getIndex() {
        LoginWxUser loginWxUser = SecurityUtils.getLoginWxUser();
        if (loginWxUser == null) {
            throw new SecurityException("请登录");
        }
        WxUser loginWxUserWxUser = loginWxUser.getWxUser();
        WxUser wxUser = wxUserMapper.selectWxUserById(loginWxUserWxUser.getId());
        WxUserVo userVo = new WxUserVo();
        BeanUtils.copyBeanProp(userVo, wxUser);
        Map<String, Object> data = new HashMap<>();
        // 积分情况
        List<WxCredit> credits = wxCreditMapper.selectWxCreditList(new WxCredit());
        Date start = DateUtil.beginOfDay(new Date());
        Date end = DateUtil.endOfDay(new Date());
        List<Long> ids = wxCreditMapper.selectWxCreditUser(wxUser.getWxOpenid(), start, end);
        WxCreditVo wxCreditVo = new WxCreditVo();
        wxCreditVo.setCredits(credits);
        wxCreditVo.setRemoveCreateIds(ids);
        // 优惠卷情况
        List<WxCoupons> wxCoupons = couponsMapper.selectWxCouponsList(new WxCoupons());
        // 用户信息
        data.put("userVo", userVo);
        // 积分信息
        data.put("credits", wxCreditVo);
        // 优惠卷信息
        data.put("wxCoupons", wxCoupons);
        return AjaxResult.success(data);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addCredit(CreditDto creditDto) {
        LoginWxUser loginWxUser = SecurityUtils.getLoginWxUser();
        if (loginWxUser == null) {
            throw new SecurityException("请登录");
        }
        WxUser loginWxUserWxUser = loginWxUser.getWxUser();
        log.info("用户信息：{}", loginWxUserWxUser.toString());
        WxUser wxUser = wxUserMapper.selectWxUserById(loginWxUserWxUser.getId());
        wxCreditMapper.insertWxCreditUser(wxUser.getWxOpenid(), creditDto.getId(), new Date());
        log.info("用户积分详情：{}, 添加积分：{}", wxUser.getPoints(), creditDto.getCreditNum());
        wxUser.setPoints(wxUser.getPoints() + creditDto.getCreditNum());
        int result = wxUserMapper.updateWxUser(wxUser);
        return result > 0;
    }

    @Override
    public List<UserCouponsVo> getCoupons() {
        String wxOpenid = SecurityUtils.getLoginWxUser().getWxUser().getWxOpenid();
        UserCouponsDto dto = new UserCouponsDto();
        dto.setOpenId(wxOpenid);
        dto.setFindDate(DateUtil.date());
        return couponsMapper.selectUserCoupons(dto);
    }

    @Override
    public boolean receiveCoupons(UserCouponsDto dto) {
        String wxOpenid = SecurityUtils.getLoginWxUser().getWxUser().getWxOpenid();
        dto.setOpenId(wxOpenid);
        WxCoupons wxCoupons = couponsMapper.selectWxCouponsById(dto.getCouponsId());
        dto.setFindDate(wxCoupons.getDeadTime());
        dto.setCreateTime(new Date());
        return couponsMapper.insertUserCoupons(dto);
    }

    @Override
    public List<CreditVo> getCredit() {
        LoginWxUser loginWxUser = SecurityUtils.getLoginWxUser();
        if (loginWxUser == null) {
            throw new SecurityException("请登录");
        }
        WxUser wxUser = loginWxUser.getWxUser();
        return wxCreditMapper.selectWxCreditByOpenId(wxUser.getWxOpenid());
    }

    @Override
    public String conCredit(String cash) {
        String result = "兑换成功";
        LoginWxUser loginWxUser = SecurityUtils.getLoginWxUser();
        if (loginWxUser == null) {
            throw new SecurityException("请登录");
        }
        WxUser loginWxUserWxUser = loginWxUser.getWxUser();
        WxUser wxUser = wxUserMapper.selectWxUserById(loginWxUserWxUser.getId());
        if (wxUser.getPoints() < 200) {
            log.error("积分不足:{}", wxUser.getPoints());
            throw new RuntimeException("积分不足");
        }
        wxUser.setVipStartTime(null == wxUser.getVipStartTime() ? LocalDate.now(ZoneId.of("Asia/Shanghai")) : wxUser.getVipStartTime());
        switch (cash) {
            case "month":
                wxUser.setPoints(wxUser.getPoints() - 200);
                wxUser.setVipLv(1);
                wxUser.setVipEndTime(wxUser.getVipStartTime().plusMonths(1));
                break;
            case "quarter":
                if (wxUser.getPoints() < 500) {
                    result = "兑换失败";
                    break;
                }
                wxUser.setPoints(wxUser.getPoints() - 500);
                wxUser.setVipLv(2);
                wxUser.setVipEndTime(wxUser.getVipStartTime().plusMonths(3));
                break;
            case "year":
                if (wxUser.getPoints() < 1800) {
                    result = "兑换失败";
                    break;
                }
                wxUser.setPoints(wxUser.getPoints() - 1800);
                wxUser.setVipLv(3);
                wxUser.setVipEndTime(wxUser.getVipStartTime().plusMonths(12));
                break;
            default:
                break;
        }

        wxUserMapper.updateWxUser(wxUser);
        return result;
    }
}
